@@ -44,7 +44,8 @@ typedef enum
   GKSU_CONTEXT_ERROR_PIPE,
   GKSU_CONTEXT_ERROR_PIPEREAD,
   GKSU_CONTEXT_ERROR_WRONGPASS,
-  GKSU_CONTEXT_ERROR_CHILDFAILED
+  GKSU_CONTEXT_ERROR_CHILDFAILED,
+  GKSU_CONTEXT_ERROR_NOT_ALLOWED
 } GksuContextError;
 
 #define GKSU_TYPE_CONTEXT_ERROR (gksu_context_error_get_type ())
@@ -98,6 +99,9 @@ struct _GksuContextClass
   GObjectClass parent;
 };
 
+typedef gboolean (*GksuAskPasswordFunc) (GksuContext*, gchar*, gpointer, GError**);
+typedef void (*GksuPasswordNotNeededFunc) (GksuContext*, gpointer);
+
 /* 
    Context handling
 */
@@ -159,15 +163,36 @@ gksu_context_get_type (void);
 gboolean
 gksu_context_try_need_password (GksuContext *context);
 
+gboolean
+gksu_context_run_full (GksuContext *context, 
+		       GksuAskPasswordFunc ask_pass, gpointer user_data, 
+		       GksuPasswordNotNeededFunc pass_not_needed, 
+		       gpointer pnn_user_data, GError **error);
+
 gboolean
 gksu_context_run (GksuContext *context, GError **error);
 
 gboolean
 gksu_context_sudo_try_need_password (GksuContext *context);
 
+gboolean
+gksu_context_sudo_run_full (GksuContext *context, 
+			    GksuAskPasswordFunc ask_pass, gpointer user_data, 
+			    GksuPasswordNotNeededFunc pass_not_needed, 
+			    gpointer pnn_user_data, GError **error);
+
 gboolean
 gksu_context_sudo_run (GksuContext *context, GError **error);
 
+/* DO NOT USE - Deprecated */
+gboolean
+gksu_context_ask_and_run (GksuContext *context, GksuAskPasswordFunc ask_pass, 
+			  gpointer user_data, GError **error);
+
+gboolean
+gksu_context_sudo_ask_and_run (GksuContext *context, GksuAskPasswordFunc ask_pass, 
+			       gpointer user_data, GError **error);
+
 G_END_DECLS
 
 #endif
