@@ -49,7 +49,7 @@ void
 clean_dir (const gchar *dirname)
 {
   gchar *xauthname;
-  
+
   xauthname = g_strdup_printf ("%s/.Xauthority", dirname);
   unlink (xauthname);
   g_free (xauthname);
@@ -66,15 +66,17 @@ clean_dir (const gchar *dirname)
 int
 main (int argc, char **argv)
 {
-  gchar *command;
+  gchar *command = NULL;
 
   gchar *xauth_dirtemplate = g_strdup ("/tmp/" PACKAGE_NAME "-XXXXXX");
 
-  gchar *xauth_dir;
-  gchar *xauth_file;
+  gchar *xauth_bin = NULL;
 
-  gchar *xauth_display;
-  gchar *xauth_token;
+  gchar *xauth_dir = NULL;
+  gchar *xauth_file = NULL;
+
+  gchar *xauth_display = NULL;
+  gchar *xauth_token = NULL;
 
   gchar buffer[255];
 
@@ -125,17 +127,31 @@ main (int argc, char **argv)
     fwrite (xauth_token, sizeof (gchar), strlen (xauth_token), file);
     fclose (file);
     chmod (tmpfilename, S_IRUSR|S_IWUSR);
-    
+
     setenv ("XAUTHORITY", xauth_file, TRUE);
 
+    /* find out where the xauth binary is located */
+    if (g_file_test ("/usr/bin/xauth", G_FILE_TEST_IS_EXECUTABLE))
+      xauth_bin = "/usr/bin/xauth";
+    else if (g_file_test ("/usr/X11R6/bin/xauth", G_FILE_TEST_IS_EXECUTABLE))
+      xauth_bin = "/usr/X11R6/bin/xauth";
+    else
+      {
+	fprintf (stderr,
+		 _("Failed to obtain xauth key: xauth binary not found "
+		   "at usual locations"));
+
+	return 1;
+      }
     command =
-      g_strdup_printf ("/usr/X11R6/bin/xauth add %s . \"`cat %s.tmp`\""
-		       " > /dev/null 2>&1", xauth_display, xauth_file);
+      g_strdup_printf ("%s add %s . \"`cat %s.tmp`\""
+		       " > /dev/null 2>&1", xauth_bin,
+		       xauth_display, xauth_file);
 
     system (command);
 
     return_code = system (argv[1]);
-    
+
     clean_dir (xauth_dir);
     return return_code;
   }
