@@ -24,6 +24,23 @@
 
 #include "gksu.h"
 
+gboolean
+su_ask_pass (GksuContext *context, gchar *prompt, gpointer data, GError **error)
+{
+  char *password;
+
+  password = getpass (prompt);
+  gksu_context_set_password (context, password);
+
+  return 0;
+}
+
+void
+password_not_needed (GksuContext *context, gpointer data)
+{
+  fprintf (stderr, "Will run %s as %s!\n", context->command, context->user);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -58,6 +75,13 @@ main (int argc, char **argv)
       gksu_context_run (context, &error);
       if (error)
 	fprintf (stderr, "gksu_run failed: %s\n", error->message);
+
+      gksu_context_set_password (context, NULL);
+      printf ("Testing gksu_ask_and_run...\n");
+      gksu_context_run_full (context, su_ask_pass, NULL, 
+			     password_not_needed, NULL, 
+			     &error);
+      gksu_context_set_password (context, NULL);
     }
   
   if (try_sudo)
@@ -77,6 +101,14 @@ main (int argc, char **argv)
       gksu_context_sudo_run (context, &error);
       if (error)
 	fprintf (stderr, "gksu_run failed: %s\n", error->message);
+
+      error = NULL;
+      gksu_context_set_password (context, NULL);
+      gksu_context_sudo_run_full (context, su_ask_pass, NULL, 
+				  password_not_needed, NULL, 
+				  &error);
+      if (error)
+	fprintf (stderr, "gksu_run failed: %s\n", error->message);
     }
       
   return 0;
